@@ -1,20 +1,19 @@
 """
-Rotator class to perform various
-rotations of factor loading matrices.
+Class to perform various rotations of factor loading matrices.
 
 :author: Jeremy Biggs (jbiggs@ets.org)
-:date: 05/21/2018
-:organization: ETS
+:author: Nitin Madnani (nmadnani@ets.org)
+:organization: Educational Testing Service
+:date: 2021-10-18
 """
 
 import numpy as np
 import scipy as sp
-
 from sklearn.base import BaseEstimator
 
-ORTHOGONAL_ROTATIONS = ['varimax', 'oblimax', 'quartimax', 'equamax']
+ORTHOGONAL_ROTATIONS = ['varimax', 'oblimax', 'quartimax', 'equamax', 'geomin_ort']
 
-OBLIQUE_ROTATIONS = ['promax', 'oblimin', 'quartimin']
+OBLIQUE_ROTATIONS = ['promax', 'oblimin', 'quartimin', 'geomin_obl']
 
 POSSIBLE_ROTATIONS = ORTHOGONAL_ROTATIONS + OBLIQUE_ROTATIONS
 
@@ -36,6 +35,8 @@ class Rotator(BaseEstimator):
             (e) quartimin (oblique rotation)
             (f) quartimax (orthogonal rotation)
             (g) equamax (orthogonal rotation)
+            (h) geomin_obl (oblique rotation)
+            (i) geomin_ort (orthogonal rotation)
         Defaults to 'varimax'.
     normalize : bool or None, optional
         Whether to perform Kaiser normalization
@@ -56,6 +57,10 @@ class Rotator(BaseEstimator):
         The gamma level for the oblimin objective.
         Ignored if the method is not 'oblimin'.
         Defaults to 0.
+    delta : float, optional
+        The delta level for geomin objectives.
+         Ignored if the method is not 'geomin_*'.
+        Defaults to 0.01.
     max_iter : int, optional
         The maximum number of iterations.
         Used for varimax and oblique rotations.
@@ -112,6 +117,7 @@ class Rotator(BaseEstimator):
                  power=4,
                  kappa=0,
                  gamma=0,
+                 delta=0.01,
                  max_iter=500,
                  tol=1e-5):
 
@@ -120,6 +126,7 @@ class Rotator(BaseEstimator):
         self.power = power
         self.kappa = kappa
         self.gamma = gamma
+        self.delta = delta
         self.max_iter = max_iter
         self.tol = tol
 
@@ -191,9 +198,9 @@ class Rotator(BaseEstimator):
                 The value of the criterion for the objective.
         """
         X = np.dot(loadings**2, np.eye(loadings.shape[1]) != 1)
-        if (0 != self.gamma):
+        if (self.gamma != 0):
             p = loadings.shape[0]
-            X = np.diag(1, p) - np.dot(np.zeros((p, p)), X)
+            X = np.diag(np.full(1, p)) - np.dot(np.zeros((p, p)), X)
         gradient = loadings * X
         criterion = np.sum(loadings**2 * X) / 4
         return {'grad': gradient, 'criterion': criterion}
@@ -256,6 +263,35 @@ class Rotator(BaseEstimator):
         criterion = f1 + f2
         return {'grad': gradient, 'criterion': criterion}
 
+    def _geomin_obj(self, loadings):
+        """
+        Geomin function objective
+
+        Parameters
+        ----------
+        loadings : array-like
+            The loading matrix
+
+        Returns
+        -------
+        gradient_dict : dict
+            A dictionary with
+            - grad : np.array
+                The gradient.
+            - criterion : float
+                The value of the criterion for the objective.
+        """
+        p, k = loadings.shape
+
+        loadings2 = loadings**2 + self.delta
+
+        pro = np.exp(np.log(loadings2).sum(1) / k)
+        rep = np.repeat(pro, k, axis=0).reshape(p, k)
+
+        gradient = (2 / k) * (loadings / loadings2) * rep
+        criterion = np.sum(pro)
+        return {'grad': gradient, 'criterion': criterion}
+
     def _oblique(self, loadings, method):
         """
         A generic function for performing
@@ -286,6 +322,8 @@ class Rotator(BaseEstimator):
             objective = self._oblimin_obj
         elif method == 'quartimin':
             objective = self._quartimin_obj
+        elif method == 'geomin_obl':
+            objective = self._geomin_obj
 
         # initialize the rotation matrix
         n_rows, n_cols = loadings.shape
@@ -370,6 +408,8 @@ class Rotator(BaseEstimator):
             objective = self._quartimax_obj
         elif method == 'equamax':
             objective = self._equamax_obj
+        elif method == 'geomin_ort':
+            objective = self._geomin_obj
 
         arr = loadings.copy()
 
@@ -462,9 +502,10 @@ class Rotator(BaseEstimator):
             # and rotation matrix
             basis = np.dot(X, rotation_mtx)
 
-            # transform data for singular value decomposition
-            transformed = np.dot(X.T, basis**3 - (1.0 / n_rows) *
-                                 np.dot(basis, np.diag(np.diag(np.dot(basis.T, basis)))))
+            # transform data for singular value decomposition using updated formula :
+            # B <- t(x) %*% (z^3 - z %*% diag(drop(rep(1, p) %*% z^2))/p)
+            diagonal = np.diag(np.squeeze(np.repeat(1, n_rows).dot(basis**2)))
+            transformed = X.T.dot(basis**3 - basis.dot(diagonal) / n_rows)
 
             # perform SVD on
             # the transformed matrix
@@ -475,7 +516,7 @@ class Rotator(BaseEstimator):
             d = np.sum(S)
 
             # check convergence
-            if old_d != 0 and d / old_d < 1 + self.tol:
+            if d < old_d * (1 + self.tol):
                 break
 
         # take inner product of loading matrix
